import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:hiflex/hiflex_device.dart';
import 'package:hiflex/hiflex_manager.dart';

void main() {
  runApp(MyApp());
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  double _sensorValue = 0.0;
  double _deltaTime = 0;

  HiflexManager _manager = new HiflexManager();
  HiflexDevice _device;
  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await HiflexManager.platformVersion;
      if (await _manager.init()) {
        _device = await _manager.findDevice();
        if (_device != null) {
          print('Start Connecting');
          if (await _device.connect()) {
            print('BT Connected');

            _device.samplingStream()?.listen((event) {
              setState(() {
                _deltaTime = event.deltaTime;
                _sensorValue = event.stretch;
              });
            });

            platformVersion = "Detect Device: ${_device.name}";
          }
        }
      }
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Text(
              'Running on: $_platformVersion\nSensor: $_sensorValue\nDeltaTime: $_deltaTime ms\n'),
        ),
      ),
    );
  }
}
