import 'dart:async';
import 'dart:typed_data';

import 'package:flutter/services.dart';
import 'package:flutter_ble_lib/flutter_ble_lib.dart';
import 'package:hiflex/hiflex_sampling.dart';

class HiflexDevice {
  String get identifier => "Unknown";
  String get name => "";

  Future<bool> connect() async => false;
  Future<void> disconnect() async {}

  Future<SamplingData> get sampling async => null;
  Stream<SamplingData> samplingStream() => null;
}

class HiflexDeviceBT implements HiflexDevice {
  static final String btSerialService = "6e400000-b5a3-f393-e0a9-e50e24dcca9e";
  static final String btTxCharacteristic =
      "6e400002-b5a3-f393-e0a9-e50e24dcca9e";
  static final String btRxCharacteristic =
      "6e400003-b5a3-f393-e0a9-e50e24dcca9e";

  String deviceName;
  int rssi;
  Peripheral peripheral;
  AdvertisementData advertisementData;

  HiflexSampling _sampling = new HiflexSampling();

  Service _serialService;
  Characteristic _serialTx;
  Characteristic _serialRx;

  StreamController<SamplingData> _sc = StreamController();

  HiflexDeviceBT(
      this.deviceName, this.rssi, this.peripheral, this.advertisementData);

  String get identifier => peripheral.identifier ?? "Unknown";
  String get name => deviceName;

  Future<bool> connect() async {
    Future<Service> findService(String targetUUID) async {
      for (var service in await peripheral.services()) {
        print("Service: {${service.uuid}}");
        if (service.uuid.compareTo(targetUUID) == 0) {
          return service;
        }
      }
      return null;
    }

    Future<Characteristic> findCharacteristic(String targetUUID) async {
      for (var characteristic in await _serialService.characteristics()) {
        if (characteristic.uuid == targetUUID) {
          return characteristic;
        }
      }
      return null;
    }

    await peripheral.connect();
    if (await peripheral.isConnected()) {
      await peripheral.discoverAllServicesAndCharacteristics();

      for (var service in await peripheral.services()) {
        print("BLE Service: {${service.uuid}}");
        for (var char in await service.characteristics()) {
          print("BLE Characteristic: {${char.uuid}}");
        }
      }

      _serialService = await findService(btSerialService);
      if (_serialService != null) {
        _serialTx = await findCharacteristic(btTxCharacteristic);
        _serialRx = await findCharacteristic(btRxCharacteristic);

        _serialRx.monitor().listen((samplingRawData) {
          _sc.add(_sampling.load(samplingRawData));
        });
        return true;
      }
    }
    return false;
  }

  Future<void> disconnect() async => peripheral.disconnectOrCancelConnection();

  Future<SamplingData> get sampling {
    return _sc.stream.first;
  }

  Stream<SamplingData> samplingStream() => _sc.stream;
}

/***
 * IOS 정리
 * https://stackoverflow.com/questions/28337159/how-to-list-discoverable-bluetooth-devices-and-already-paired-devices-in-ios-wh
 * 
 * 
 */
