import 'dart:async';
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:flutter_ble_lib/flutter_ble_lib.dart';
import 'package:permission_handler/permission_handler.dart';

import 'package:hiflex/hiflex_device.dart';

class HiflexManager {
  static const MethodChannel _channel = const MethodChannel('hiflex');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  static final HiflexManager _instance = new HiflexManager._internal();

  factory HiflexManager() {
    return _instance;
  }

  HiflexManager._internal();

  BleManager _manager = BleManager();
  List<HiflexDevice> deviceList = [];

  final StreamController<HiflexDevice> _sc = StreamController();

  Future<bool> init() async {
    checkPermissions() async {
      if (Platform.isAndroid) {
        if (await Permission.contacts.request().isGranted) {}
        Map<Permission, PermissionStatus> statuses =
            await [Permission.location].request();
        print("Location Permission: ${statuses[Permission.location]}");
      }
    }

    print("Hiflex Manager Init - BLE Manager");
    await _manager
        .createClient(
            restoreStateIdentifier: "hiflex.restore.state.identifier",
            restoreStateAction: (peripherals) {
              peripherals?.forEach((peripheral) {
                print("Restored peripheral: ${peripheral.name}");
              });
            })
        .catchError((e) => print("Couldn't create BLE client  $e"))
        .then((_) => checkPermissions()) //BLE 생성 후 퍼미션 체크
        .catchError((e) => print("Permission check error $e"));

    return true;
  }

  Future<HiflexDevice> findDevice() async {
    if (await _manager.bluetoothState() == BluetoothState.POWERED_ON) {
      print("Start Bluetooth Scanning");

      _manager.startPeripheralScan().listen((scanResult) async {
        var name = scanResult.peripheral.name ??
            scanResult.advertisementData.localName ??
            "Unknown";

        var findDevice = deviceList.any((element) =>
            element.identifier == scanResult.peripheral.identifier);

        print("Scan Device: $name");
        if (!findDevice) {
          var services = scanResult.advertisementData.serviceUuids ?? [];
          var findService = services
              .any((element) => element == HiflexDeviceBT.btSerialService);
          if (findService) {
            var newDevice = HiflexDeviceBT(name, scanResult.rssi,
                scanResult.peripheral, scanResult.advertisementData);
            deviceList.add(newDevice);
            _sc.add(newDevice);

            _manager.stopPeripheralScan();
          }
        }
      });
      return await _sc.stream.first;
    }
    return null;
  }
}
